module rgbcalc(clk, scr, lives, x, y, ballx, bally, bricks, paddlex, r, g, b);

input [27:0] scr;
input [1:0] lives;
input clk;
input [9:0] x, y;
input [9:0] ballx;
input [8:0] bally;
input [59:0] bricks;
input [9:0] paddlex;
output r, g, b;

wire inpaddle, inball, inlives;
wire rb,ra,gb,ga,bb,ba;
wire r, g, b;
wire [7:0] bnum, ynum;
wire [5:0] bricknum;
wire bvalid;

// stuff used for the 7 segment score display
wire inscore;
wire [1:0] i;
wire [3:0] xpos;
wire [4:0] ypos;

assign i = 3 - x[5:4];
assign xpos = x[3:0];
assign ypos = y[4:0];

// this signal goes high if we are in a pixel that is in one of the 'lit' segments of the 4-digit 7 segment display
assign inscore = ((xpos < 14) && (x[9:6] == 8) && (y[8:5] == 0) && ((scr[i*7] && (ypos <= 2)) || (scr[i*7+1] && (xpos >= 11) && (ypos <= 16)) || (scr[i*7+2] && (xpos >= 11) && (ypos >= 16)) || (scr[i*7+3] && (ypos >= 29)) || (scr[i*7+4] && (xpos <= 2) && (ypos >= 16)) || (scr[i*7+5] && (xpos <= 2) && (ypos <= 16)) || (scr[i*7+6] && (ypos >= 14) && (ypos <= 17))));

// this signal goes high if we are in the area to display the number of lives
assign inlives = (y < 5 && ((x >= 5 && x <= 8 && lives != 0) || (x >= 10 && x <= 13 && lives > 1) || (x >= 15 && x <= 18 && lives == 3))) ? 1 : 0;

// are we in a pixel that is inside the paddle?
assign inpaddle = (y >= 460 && y <= 465 && x >= paddlex - 30 && x <= paddlex + 30);

// are we in a pixel that is inside the ball?
assign inball = (x >= ballx - 2 && x <= ballx + 2 && y >= bally - 2 && y <= bally + 2); 

// figure out if we are in a brick
assign ynum = y[8:5];
assign bnum = (ynum >= 3 && ynum <= 8) ? (10 * ynum + x[9:6] - 29) : 0;
assign bricknum = (bnum > 60 || bnum == 0) ? 0 : bnum; // != 0 if we are actually in a brick
assign bvalid = (bricknum == 0) ? 0 : bricks[bricknum-1];

// CALCULATE RGB
//
//
// Here is the algorithm to determine r.  g and b are similar in structure.  Basically, we have a precedence
// set so that certain things appear in the foreground first... the precedence order is: ball, paddle, score, brick, bg
// if (in the ball, in the paddle, or in the score)
//   r = 1;
// else if (in the top two rows or bottom two rows of bricks, and the brick is there)
//   r = 1;
// else
//   r = 0;
assign rb = (bvalid && ((bricknum >= 1 && bricknum <= 20) || (bricknum >= 51 && bricknum <= 60))) ? 1 : 0;
assign ra = (inpaddle | inscore) ? 1 : rb;
assign r = (inball) ? 1 : ra;

assign gb = (bvalid && (bricknum >= 11 && bricknum <= 40)) ? 1 : 0;
assign ga = (inpaddle | inscore) ? 1 : gb;
assign g = (inball) ? 0 : ga;

assign bb = (bvalid && (bricknum >= 31 && bricknum <= 60)) ? 1 : 0;
assign ba = (inpaddle | inlives | inscore) ? 1 : bb;
assign b = (inball) ? 0 : ba;

endmodule
